/*
 * Copyright (c) 2008 Jiri Svoboda
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup rtld rtld
 * @brief
 * @{
 */ 
/**
 * @file
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <rtld.h>
#include <symbol.h>
#include <elf.h>

static elf_symbol_t *def_find_in_module(char *name, module_t *m)
{
	elf_symbol_t *sym_table;
	elf_symbol_t *sym;
	elf_word nchain;
	size_t i;
	char *s_name;
	char *module_name;

	module_name = m->dyn.soname;
//	printf("def_find_in_module('%s', %s)\n", name, module_name);

	sym_table = m->dyn.sym_tab;
	nchain = m->dyn.hash[1];

	for (i = 0; i < nchain; ++i) {
		sym = &sym_table[i];
		s_name = m->dyn.str_tab + sym->st_name;
//		printf("cmp sym '%s'\n", s_name);
		if (strcmp(name, s_name) == 0) {
//			printf("name match\n");
			if (sym->st_shndx != SHN_UNDEF) {
//				printf("definition found. idx=%d\n", i);
				return sym;
			} else {
//				printf("not a definition\n");
			}
		}
	}

//	printf("not found\n");
	return NULL;
}


elf_symbol_t *symbol_def_find(char *name, module_t **m)
{
	elf_symbol_t *sym;

	sym = def_find_in_module(name, runtime_env.program);
	if (sym != NULL) { *m = runtime_env.program; return sym; }	

	sym = def_find_in_module(name, runtime_env.libc);
	if (sym != NULL) { *m = runtime_env.libc; return sym; }

	sym = def_find_in_module(name, &runtime_env.rtld);
	if (sym != NULL) { *m = &runtime_env.rtld; return sym; }

	printf("symbol found nowhere\n");
	*m = NULL;
	return NULL;
}

/** @}
 */
